/*
 * Copyright 2019 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.gaellalire.vestige.spi.resolver.maven;

import fr.gaellalire.vestige.spi.job.JobHelper;
import fr.gaellalire.vestige.spi.resolver.ResolverException;
import fr.gaellalire.vestige.spi.resolver.Scope;

/**
 * @author Gael Lalire
 * @since 2.0
 */
public interface CreateClassLoaderConfigurationRequest {

    ModifyLoadedDependencyRequest addModifyLoadedDependency(String groupId, String artifactId);

    ModifyLoadedDependencyRequest addModifyLoadedDependency(String groupId, String artifactId, String classifier);

    void addModifyScope(String groupId, String artifactId, Scope scope);

    void setNamedModuleActivated(boolean namedModuleActivated);

    void addReads(String source, String target);

    void addExports(String source, String pn, String target);

    void addOpens(String source, String pn, String target);

    void setSelfExcluded(boolean selfExcluded);

    void setDependenciesExcluded(boolean dependenciesExcluded);

    /**
     * The dependency is excluded and its children added to its parent.
     * @since 3.1
     */
    void addExclude(String groupId, String artifactId);

    /**
     * The dependency is excluded and its children added to its parent.
     * @since 3.3
     */
    void addExclude(String groupId, String artifactId, String extension, String classifier);

    /**
     * The dependency and its parents are excluded, its children added to the root.
     * @since 3.1
     */
    void addExcludeWithParents(String groupId, String artifactId);

    /**
     * The dependency and its parents are excluded, its children added to the root.
     * @since 3.3
     */
    void addExcludeWithParents(String groupId, String artifactId, String extension, String classifier);

    MavenResolvedClassLoaderConfiguration execute() throws ResolverException;

    /**
     * @since 3.3
     */
    MavenResolvedClassLoaderConfiguration execute(JobHelper jobHelper) throws ResolverException;

}
